@@ -1,8 +1,8 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qdatastream.h"
-#include "qdatastream_p.h"
 
 #if !defined(QT_NO_DATASTREAM) || defined(QT_BOOTSTRAPPED)
 #include "qbuffer.h"
@@ -13,6 +13,8 @@
 #include <stdlib.h>
 #include "qendian.h"
 
+#include <QtCore/q20memory.h>
+
 QT_BEGIN_NAMESPACE
 
 constexpr quint32 QDataStream::NullCode;
@@ -70,17 +72,30 @@ constexpr quint32 QDataStream::ExtendedSize;
     need of manually defining streaming operators. Enum classes are
     serialized using the declared size.
 
-    To take one example, a \c{char *} string is written as a 32-bit
-    integer equal to the length of the string including the '\\0' byte,
-    followed by all the characters of the string including the
-    '\\0' byte. When reading a \c{char *} string, 4 bytes are read to
-    create the 32-bit length value, then that many characters for the
-    \c {char *} string including the '\\0' terminator are read.
-
     The initial I/O device is usually set in the constructor, but can be
     changed with setDevice(). If you've reached the end of the data
     (or if there is no I/O device set) atEnd() will return true.
 
+    \section1 Serializing containers and strings
+
+    The serialization format is a length specifier first, then \a l bytes of data.
+    The length specifier is one quint32 if the version is less than 6.7 or if the
+    number of elements is less than 0xfffffffe (2^32 -2). Otherwise there is
+    an extend value 0xfffffffe followed by one quint64 with the actual value.
+    In addition for containers that support isNull(), it is encoded as a single
+    quint32 with all bits set and no data.
+
+    To take one example, if the string size fits into 32 bits, a \c{char *} string
+    is written as a 32-bit integer equal to the length of the string, including
+    the '\\0' byte, followed by all the characters of the string, including the
+    '\\0' byte. If the string size is greater, the value 0xffffffffe is written
+    as a marker of an extended size, followed by 64 bits of the actual size.
+    When reading a \c {char *} string, 4 bytes are read first. If the value is
+    not equal to 0xffffffffe (the marker of extended size), then these 4 bytes
+    are treated as the 32 bit size of the string. Otherwise, the next 8 bytes are
+    read and treated as a 64 bit size of the string. Then, all the characters for
+    the \c {char *} string, including the '\\0' terminator, are read.
+
     \section1 Versioning
 
     QDataStream's binary format has evolved since Qt 1.0, and is
@@ -258,7 +273,7 @@ constexpr quint32 QDataStream::ExtendedSize;
         return retVal;
 
 #define CHECK_STREAM_TRANSACTION_PRECOND(retVal) \
-    if (!d || d->transactionDepth == 0) { \
+    if (transactionDepth == 0) { \
         qWarning("QDataStream: No transaction in progress"); \
         return retVal; \
     }
@@ -271,12 +286,6 @@ constexpr quint32 QDataStream::ExtendedSize;
 
 QDataStream::QDataStream()
 {
-    dev = nullptr;
-    owndev = false;
-    byteorder = BigEndian;
-    ver = Qt_DefaultCompiledVersion;
-    noswap = QSysInfo::ByteOrder == QSysInfo::BigEndian;
-    q_status = Ok;
 }
 
 /*!
@@ -288,11 +297,6 @@ QDataStream::QDataStream()
 QDataStream::QDataStream(QIODevice *d)
 {
     dev = d;                                // set device
-    owndev = false;
-    byteorder = BigEndian;                        // default byte order
-    ver = Qt_DefaultCompiledVersion;
-    noswap = QSysInfo::ByteOrder == QSysInfo::BigEndian;
-    q_status = Ok;
 }
 
 /*!
@@ -317,10 +321,6 @@ QDataStream::QDataStream(QByteArray *a, OpenMode flags)
     buf->open(flags);
     dev = buf;
     owndev = true;
-    byteorder = BigEndian;
-    ver = Qt_DefaultCompiledVersion;
-    noswap = QSysInfo::ByteOrder == QSysInfo::BigEndian;
-    q_status = Ok;
 }
 
 /*!
@@ -341,10 +341,6 @@ QDataStream::QDataStream(const QByteArray &a)
     buf->open(QIODevice::ReadOnly);
     dev = buf;
     owndev = true;
-    byteorder = BigEndian;
-    ver = Qt_DefaultCompiledVersion;
-    noswap = QSysInfo::ByteOrder == QSysInfo::BigEndian;
-    q_status = Ok;
 }
 
 /*!
@@ -406,16 +402,14 @@ bool QDataStream::atEnd() const
 }
 
 /*!
+    \fn QDataStream::FloatingPointPrecision QDataStream::floatingPointPrecision() const
+
     Returns the floating point precision of the data stream.
 
     \since 4.6
 
     \sa FloatingPointPrecision, setFloatingPointPrecision()
 */
-QDataStream::FloatingPointPrecision QDataStream::floatingPointPrecision() const
-{
-    return d ? d->floatingPointPrecision : QDataStream::DoublePrecision;
-}
 
 /*!
     Sets the floating point precision of the data stream to \a precision. If the floating point precision is
@@ -439,22 +433,17 @@ QDataStream::FloatingPointPrecision QDataStream::floatingPointPrecision() const
 */
 void QDataStream::setFloatingPointPrecision(QDataStream::FloatingPointPrecision precision)
 {
-    if (!d)
-        d.reset(new QDataStreamPrivate());
-    d->floatingPointPrecision = precision;
+    fpPrecision = precision;
 }
 
 /*!
+    \fn QDataStream::status() const
+
     Returns the status of the data stream.
 
     \sa Status, setStatus(), resetStatus()
 */
 
-QDataStream::Status QDataStream::status() const
-{
-    return q_status;
-}
-
 /*!
     Resets the status of the data stream.
 
@@ -502,11 +491,14 @@ void QDataStream::setStatus(Status status)
 
 void QDataStream::setByteOrder(ByteOrder bo)
 {
+#if QT_VERSION < QT_VERSION_CHECK(7, 0, 0) && !defined(QT_BOOTSTRAPPED)
+    // accessed by inline byteOrder() prior to Qt 6.8
     byteorder = bo;
+#endif
     if (QSysInfo::ByteOrder == QSysInfo::BigEndian)
-        noswap = (byteorder == BigEndian);
+        noswap = (bo == BigEndian);
     else
-        noswap = (byteorder == LittleEndian);
+        noswap = (bo == LittleEndian);
 }
 
 
@@ -556,6 +548,8 @@ void QDataStream::setByteOrder(ByteOrder bo)
     \value Qt_6_5 Same as Qt_6_0
     \value Qt_6_6 Version 21 (Qt 6.6)
     \value Qt_6_7 Version 22 (Qt 6.7)
+    \value Qt_6_8 Same as Qt_6_7
+    \value Qt_6_9 Same as Qt_6_7
     \omitvalue Qt_DefaultCompiledVersion
 
     \sa setVersion(), version()
@@ -629,10 +623,7 @@ void QDataStream::startTransaction()
 {
     CHECK_STREAM_PRECOND(Q_VOID)
 
-    if (!d)
-        d.reset(new QDataStreamPrivate());
-
-    if (++d->transactionDepth == 1) {
+    if (++transactionDepth == 1) {
         dev->startTransaction();
         resetStatus();
     }
@@ -661,7 +652,7 @@ void QDataStream::startTransaction()
 bool QDataStream::commitTransaction()
 {
     CHECK_STREAM_TRANSACTION_PRECOND(false)
-    if (--d->transactionDepth == 0) {
+    if (--transactionDepth == 0) {
         CHECK_STREAM_PRECOND(false)
 
         if (q_status == ReadPastEnd) {
@@ -701,7 +692,7 @@ void QDataStream::rollbackTransaction()
     setStatus(ReadPastEnd);
 
     CHECK_STREAM_TRANSACTION_PRECOND(Q_VOID)
-    if (--d->transactionDepth != 0)
+    if (--transactionDepth != 0)
         return;
 
     CHECK_STREAM_PRECOND(Q_VOID)
@@ -737,7 +728,7 @@ void QDataStream::abortTransaction()
     q_status = ReadCorruptData;
 
     CHECK_STREAM_TRANSACTION_PRECOND(Q_VOID)
-    if (--d->transactionDepth != 0)
+    if (--transactionDepth != 0)
         return;
 
     CHECK_STREAM_PRECOND(Q_VOID)
@@ -1041,7 +1032,7 @@ QDataStream &QDataStream::operator>>(char32_t &c)
 
 #if QT_DEPRECATED_SINCE(6, 11)
 
-/*
+/*!
     \deprecated [6.11] Use an overload that takes qint64 length instead.
 */
 QDataStream &QDataStream::readBytes(char *&s, uint &l)
@@ -1098,28 +1089,24 @@ QDataStream &QDataStream::readBytes(char *&s, qint64 &l)
         return *this;
     }
 
-    qsizetype step = 1024 * 1024;
+    qsizetype step = (dev->bytesAvailable() >= len) ? len : 1024 * 1024;
     qsizetype allocated = 0;
-    char *prevBuf = nullptr;
-    char *curBuf = nullptr;
+    std::unique_ptr<char[]> curBuf = nullptr;
 
+    constexpr qsizetype StepIncreaseThreshold = std::numeric_limits<qsizetype>::max() / 2;
     do {
         qsizetype blockSize = qMin(step, len - allocated);
-        prevBuf = curBuf;
-        curBuf = new char[allocated + blockSize + 1];
-        if (prevBuf) {
-            memcpy(curBuf, prevBuf, allocated);
-            delete [] prevBuf;
-        }
-        if (readBlock(curBuf + allocated, blockSize) != blockSize) {
-            delete [] curBuf;
+        const qsizetype n = allocated + blockSize + 1;
+        if (const auto prevBuf = std::exchange(curBuf, q20::make_unique_for_overwrite<char[]>(n)))
+            memcpy(curBuf.get(), prevBuf.get(), allocated);
+        if (readBlock(curBuf.get() + allocated, blockSize) != blockSize)
             return *this;
-        }
         allocated += blockSize;
-        step *= 2;
+        if (step <= StepIncreaseThreshold)
+            step *= 2;
     } while (allocated < len);
 
-    s = curBuf;
+    s = curBuf.release();
     s[len] = '\0';
     l = len;
     return *this;
@@ -1272,18 +1259,13 @@ QDataStream &QDataStream::operator<<(qint64 i)
 */
 
 /*!
+    \fn QDataStream &QDataStream::operator<<(bool i)
+    \overload
+
     Writes a boolean value, \a i, to the stream. Returns a reference
     to the stream.
 */
 
-QDataStream &QDataStream::operator<<(bool i)
-{
-    CHECK_STREAM_WRITE_PRECOND(*this)
-    if (!dev->putChar(qint8(i)))
-        q_status = WriteFailed;
-    return *this;
-}
-
 /*!
     \overload
 
