@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qtexttable.h"
 #include "qtextcursor.h"
@@ -50,6 +14,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 /*!
     \class QTextTableCell
     \reentrant
@@ -106,10 +72,12 @@ QT_BEGIN_NAMESPACE
     Sets the cell's character format to \a format. This can for example be used to change
     the background color of the entire cell:
 
+    \code
     QTextTableCell cell = table->cellAt(2, 3);
     QTextCharFormat format = cell.format();
     format.setBackground(Qt::blue);
     cell.setFormat(format);
+    \endcode
 
     Note that the cell's row or column span cannot be changed through this function. You have
     to use QTextTable::mergeCells and QTextTable::splitCell instead.
@@ -415,7 +383,7 @@ void QTextTablePrivate::fragmentRemoved(QChar type, uint fragment)
         return;
     if (type == QTextBeginningOfFrame) {
         Q_ASSERT(cells.indexOf(int(fragment)) != -1);
-        cells.removeAll(fragment);
+        cells.removeAll(int(fragment));
         if (fragment_start == fragment && cells.size()) {
             fragment_start = cells.at(0);
         }
@@ -973,7 +941,7 @@ void QTextTable::removeColumns(int pos, int num)
     QTextTableFormat tfmt = format();
     tfmt.setColumns(tfmt.columns()-num);
     QList<QTextLength> columnWidths = tfmt.columnWidthConstraints();
-    if (columnWidths.count() > pos) {
+    if (columnWidths.size() > pos) {
         columnWidths.remove(pos, num);
         tfmt.setColumnWidthConstraints (columnWidths);
     }
@@ -1110,7 +1078,7 @@ void QTextTable::mergeCells(int row, int column, int numRows, int numCols)
                     QTextCursorPrivate::fromPosition(p, insertPos++).insertBlock();
                     p->move(pos + 1, insertPos, nextPos - pos);
                 } else if (rowHasText) {
-                    QTextCursorPrivate::fromPosition(p, insertPos++).insertText(QLatin1String(" "));
+                    QTextCursorPrivate::fromPosition(p, insertPos++).insertText(" "_L1);
                     p->move(pos + 1, insertPos, nextPos - pos);
                 } else {
                     p->move(pos, insertPos, nextPos - pos);
@@ -1335,3 +1303,5 @@ void QTextTable::setFormat(const QTextTableFormat &format)
 */
 
 QT_END_NAMESPACE
+
+#include "moc_qtexttable.cpp"
