@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 //#define QHOSTINFO_DEBUG
 
@@ -249,10 +250,7 @@ int QHostInfo::lookupHost(const QString &name, const QObject *receiver, const ch
 
 /*!
     \fn void QHostInfo::swap(QHostInfo &other)
-
-    Swaps host-info \a other with this host-info. This operation is
-    very fast and never fails.
-
+    \memberswap{host-info}
     \since 5.10
 */
 
