@@ -7,15 +7,16 @@
 #include <QObject>
 
 #include <qpa/qplatformclipboard.h>
+#include <private/qstdweb_p.h>
 #include <QMimeData>
 
 #include <emscripten/bind.h>
 #include <emscripten/val.h>
 
-#include "qwasmeventtranslator.h"
-
 QT_BEGIN_NAMESPACE
 
+struct KeyEvent;
+
 class QWasmClipboard : public QObject, public QPlatformClipboard
 {
 public:
@@ -34,10 +35,14 @@ public:
     bool supportsMode(QClipboard::Mode mode) const override;
     bool ownsMode(QClipboard::Mode mode) const override;
 
-    ProcessKeyboardResult processKeyboard(const QWasmEventTranslator::TranslatedEvent &event,
-                                          const QFlags<Qt::KeyboardModifier> &modifiers);
-    static void installEventHandlers(const emscripten::val &target);
+    ProcessKeyboardResult processKeyboard(const KeyEvent &event);
+    static void installEventHandlers(emscripten::val &target);
     bool hasClipboardApi();
+    void sendClipboardData(emscripten::val event);
+
+    static void cut(emscripten::val event);
+    static void copy(emscripten::val event);
+    static void paste(emscripten::val event);
 
 private:
     void initClipboardPermissions();
