@@ -1,31 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2012 Giuseppe D'Angelo <dangelog@gmail.com>
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2012 Giuseppe D'Angelo <dangelog@gmail.com>
+// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2024 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QLibraryInfo>
@@ -139,7 +115,7 @@ static QString doCompare(const QStringList &actual, const QStringList &expected,
                 ba.append("File " + fi.absoluteFilePath().toUtf8() + " does not exist!");
                 break;
             }
-            const quint64 timeStamp = quint64(fi.lastModified().toMSecsSinceEpoch());
+            const quint64 timeStamp = quint64(fi.lastModified(QTimeZone::UTC).toMSecsSinceEpoch());
             expectedLine.clear();
             for (int shift = 56; shift >= 0; shift -= 8) {
                 expectedLine.append(QLatin1String("0x"));
@@ -173,6 +149,14 @@ void tst_rcc::rcc_data()
     QTest::newRow("size-1") << sizesPath << "size-1.qrc" << "size-1.expected";
     QTest::newRow("size-2-0-35-1") << sizesPath << "size-2-0-35-1.qrc" <<
                                       (sizeof(size_t) == 8 ? "size-2-0-35-1.expected" : "size-2-0-35-1.expected32");
+
+    QTest::newRow("legal") << m_dataPath + QLatin1StringView("/legal")
+                               << "legal.qrc" << "rcc_legal.cpp";
+
+    if (sizeof(size_t) == 8) {
+        const QString deduplicationPath = m_dataPath + QLatin1String("/deduplication");
+        QTest::newRow("deduplication") << deduplicationPath << "deduplication.qrc" << "deduplication.expected";
+    }
 }
 
 static QStringList readLinesFromFile(const QString &fileName,
@@ -181,10 +165,8 @@ static QStringList readLinesFromFile(const QString &fileName,
     QFile file(fileName);
 
     bool ok = file.open(QIODevice::ReadOnly | QIODevice::Text);
-    if (!ok) {
-        QWARN(qPrintable(QString::fromLatin1("Could not open testdata file %1: %2")
-                         .arg(fileName, file.errorString())));
-    }
+    if (!ok)
+        qWarning() << "Could not open testdata file" << fileName << ":" << file.errorString();
 
     return QString::fromUtf8(file.readAll()).split(QLatin1Char('\n'), splitBehavior);
 }
@@ -238,8 +220,8 @@ static QStringMap readExpectedFiles(const QString &fileName)
 {
     QStringMap expectedFiles;
 
-    QStringList lines = readLinesFromFile(fileName, Qt::SkipEmptyParts);
-    foreach (const QString &line, lines) {
+    const QStringList lines = readLinesFromFile(fileName, Qt::SkipEmptyParts);
+    for (const QString &line : lines) {
         QString resourceFileName = line.section(QLatin1Char(' '), 0, 0, QString::SectionSkipEmpty);
         QString actualFileName = line.section(QLatin1Char(' '), 1, 1, QString::SectionSkipEmpty);
         expectedFiles[resourceFileName] = actualFileName;
@@ -274,8 +256,7 @@ void tst_rcc::binary_data()
     QDirIterator iter(dataPath, QStringList() << QLatin1String("*.qrc"));
     while (iter.hasNext())
     {
-        iter.next();
-        QFileInfo qrcFileInfo = iter.fileInfo();
+        QFileInfo qrcFileInfo = iter.nextFileInfo();
         QString absoluteBaseName = QFileInfo(qrcFileInfo.absolutePath(), qrcFileInfo.baseName()).absoluteFilePath();
         QString rccFileName = absoluteBaseName + QLatin1String(".rcc");
 
@@ -304,8 +285,8 @@ void tst_rcc::binary_data()
         QString localeFileName = absoluteBaseName + QLatin1String(".locale");
         QFile localeFile(localeFileName);
         if (localeFile.exists()) {
-            QStringList locales = readLinesFromFile(localeFileName, Qt::SkipEmptyParts);
-            foreach (const QString &locale, locales) {
+            const QStringList locales = readLinesFromFile(localeFileName, Qt::SkipEmptyParts);
+            for (const QString &locale : locales) {
                 QString expectedFileName = QString::fromLatin1("%1.%2.%3").arg(absoluteBaseName, locale, QLatin1String("expected"));
                 QStringMap expectedFiles = readExpectedFiles(expectedFileName);
                 QTest::newRow(qPrintable(qrcFileInfo.baseName() + QLatin1Char('_') + locale)) << rccFileName
@@ -335,10 +316,15 @@ void tst_rcc::binary()
     const QString rootPrefix = QLatin1String("/test_root/");
     const QString resourceRootPrefix = QLatin1Char(':') + rootPrefix;
 
-    QLocale oldDefaultLocale;
+    const auto restoreLocale = qScopeGuard([prior = QLocale()]() {
+        QLocale::setDefault(prior);
+    });
     QLocale::setDefault(locale);
     QVERIFY(QFile::exists(resourceFile));
     QVERIFY(QResource::registerResource(resourceFile, rootPrefix));
+    const auto unregister = qScopeGuard([resourceFile, rootPrefix]() {
+        QVERIFY(QResource::unregisterResource(resourceFile, rootPrefix));
+    });
 
     { // need to destroy the iterators on the resource, in order to be able to unregister it
 
@@ -364,7 +350,7 @@ void tst_rcc::binary()
 
     // now actually check the file contents
     QDir directory(baseDirectory);
-    for (QStringMap::const_iterator i = expectedFiles.constBegin(); i != expectedFiles.constEnd(); ++i) {
+    for (auto i = expectedFiles.constBegin(); i != expectedFiles.constEnd(); ++i) {
         QString resourceFileName = i.key();
         QString actualFileName = i.value();
 
@@ -381,9 +367,6 @@ void tst_rcc::binary()
     }
 
     }
-
-    QVERIFY(QResource::unregisterResource(resourceFile, rootPrefix));
-    QLocale::setDefault(oldDefaultLocale);
 }
 
 void tst_rcc::readback_data()
@@ -501,7 +484,7 @@ void tst_rcc::cleanupTestCase()
     QFileInfoList entries = dataDir.entryInfoList(QStringList() << QLatin1String("*.rcc"));
     QDir dataDepDir(m_dataPath + QLatin1String("/depfile"));
     entries += dataDepDir.entryInfoList({QLatin1String("*.d"), QLatin1String("*.qrc.cpp")});
-    foreach (const QFileInfo &entry, entries)
+    for (const QFileInfo &entry : std::as_const(entries))
         QFile::remove(entry.absoluteFilePath());
 }
 
